
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_BOX
#define GR_SHQN_BOX


#include "coord3d.h"
#include "coord3f.h"


class Boxd
{
  public:
    Coord3d min;                        // min coordinate point
    Coord3d max;                        // max coordinate point

    inline  Boxd( void ) { }
    inline ~Boxd( void ) { }
		// Gonfiamento/sgonfiamento box
    inline  Boxd & Inflate( const double s );
    inline  Boxd & Inflate( const Coord3d & delta );
	inline  Boxd & operator += ( const Coord3d & delta );
		// Il box contiene il solo punto p
    inline  Boxd & Set( const Coord3d & p );
	inline  Boxd & operator = ( const Coord3d & p );
		// Unione di due box
    inline  Boxd & Add( const Boxd & b );
	inline  Boxd & operator |= ( const Boxd & b );
		// Il box contiene anche il punto p
	inline  Boxd & Add( const Coord3d & p );
    inline  Boxd & operator |= ( const Coord3d & p );
		// Intersezione di box
    inline  Boxd & Intersect( const Boxd & b );
	inline  Boxd & operator &= ( const Boxd & b );
    inline  Boxd & Translate( const Coord3d & p );
    inline  bool  IsIn( Coord3d const & p ) const;
    inline  int   IsEmpty( void ) const;
    inline  double Diag( void ) const;
    inline  double Volume( void ) const;
    inline  double DimX( void ) const;
    inline  double DimY( void ) const;
    inline  double DimZ( void ) const;
};


inline Boxd & Boxd::Inflate( const double s )
{
    Inflate( (max-min)*s );
	return *this;
}


inline Boxd & Boxd::Inflate( const Coord3d & delta )
{
    min -= delta;
    max += delta;
	return *this;
}


inline Boxd & Boxd::operator += ( const Coord3d & delta )
{
    min -= delta;
    max += delta;
	return *this;
}


inline Boxd & Boxd::Set( const Coord3d & p )
{
    min = max = p;
	return *this;
}


inline Boxd & Boxd::operator = ( const Coord3d & p )
{
    min = max = p;
	return *this;
}


inline Boxd & Boxd::Add( Boxd const & b )
{
    if(min.x > b.min.x) min.x = b.min.x;
    if(min.y > b.min.y) min.y = b.min.y;
    if(min.z > b.min.z) min.z = b.min.z;

    if(max.x < b.max.x) max.x = b.max.x;
    if(max.y < b.max.y) max.y = b.max.y;
    if(max.z < b.max.z) max.z = b.max.z;
	return *this;
}


inline Boxd & Boxd::operator |= ( Boxd const & b )
{
    if(min.x > b.min.x) min.x = b.min.x;
    if(min.y > b.min.y) min.y = b.min.y;
    if(min.z > b.min.z) min.z = b.min.z;

    if(max.x < b.max.x) max.x = b.max.x;
    if(max.y < b.max.y) max.y = b.max.y;
    if(max.z < b.max.z) max.z = b.max.z;
	return *this;
}


inline Boxd & Boxd::Add( const Coord3d & p )
{
    if(min.x > p.x) min.x = p.x;
    if(min.y > p.y) min.y = p.y;
    if(min.z > p.z) min.z = p.z;

    if(max.x < p.x) max.x = p.x;
    if(max.y < p.y) max.y = p.y;
    if(max.z < p.z) max.z = p.z;
	return *this;
}


inline Boxd & Boxd::operator |= ( const Coord3d & p )
{
    if(min.x > p.x) min.x = p.x;
    if(min.y > p.y) min.y = p.y;
    if(min.z > p.z) min.z = p.z;

    if(max.x < p.x) max.x = p.x;
    if(max.y < p.y) max.y = p.y;
    if(max.z < p.z) max.z = p.z;
	return *this;
}


inline Boxd & Boxd::Intersect( const Boxd & b )
{
    if(min.x < b.min.x) min.x = b.min.x;
    if(min.y < b.min.y) min.y = b.min.y;
    if(min.z < b.min.z) min.z = b.min.z;

    if(max.x > b.max.x) max.x = b.max.x;
    if(max.y > b.max.y) max.y = b.max.y;
    if(max.z > b.max.z) max.z = b.max.z;

	if(max.x < min.x) max.x = min.x = 0;
	if(max.y < min.y) max.y = min.y = 0;
	if(max.z < min.z) max.z = min.z = 0;
	
	return *this;
}


inline Boxd & Boxd::operator &= ( const Boxd & b )
{
    if(min.x < b.min.x) min.x = b.min.x;
    if(min.y < b.min.y) min.y = b.min.y;
    if(min.z < b.min.z) min.z = b.min.z;

    if(max.x > b.max.x) max.x = b.max.x;
    if(max.y > b.max.y) max.y = b.max.y;
    if(max.z > b.max.z) max.z = b.max.z;

	if(max.x < min.x) max.x = min.x = 0;
	if(max.y < min.y) max.y = min.y = 0;
	if(max.z < min.z) max.z = min.z = 0;
	
	return *this;
}


inline Boxd & Boxd::Translate( const Coord3d & p )
{
    min += p;
    max += p;
	return *this;
}


inline bool Boxd::IsIn( Coord3d const & p ) const
{
    return (
        min.x <= p.x && p.x >= max.x &&
        min.y <= p.y && p.y >= max.y &&
        min.z <= p.z && p.z >= max.z
    );
}


inline int Boxd::IsEmpty( void ) const
{
    return min==max;
}


inline double Boxd::Diag( void ) const
{
    return Dist(min,max);
}


inline  double Boxd::Volume( void ) const
{
    return (max.x-min.x)*(max.y-min.y)*(max.z-min.z);
}


inline double Boxd::DimX( void ) const
{
    return max.x-min.x;
}


inline double Boxd::DimY( void ) const
{
    return max.y-min.y;
}


inline double Boxd::DimZ( void ) const
{
    return max.z-min.z;
}





class Boxf
{
  public:
    Coord3f min;                        // min coordinate point
    Coord3f max;                        // max coordinate point

    inline  Boxf( void ) { }
    inline ~Boxf( void ) { }
		// Gonfiamento/sgonfiamento box
    inline  Boxf & Inflate( const float s );
    inline  Boxf & Inflate( const Coord3f & delta );
	inline  Boxf & operator += ( const Coord3f & delta );
		// Il box contiene il solo punto p
    inline  Boxf & Set( const Coord3f & p );
	inline  Boxf & operator = ( const Coord3f & p );
		// Unione di due box
    inline  Boxf & Add( const Boxf & b );
	inline  Boxf & operator |= ( const Boxf & b );
		// Il box contiene anche il punto p
	inline  Boxf & Add( const Coord3f & p );
    inline  Boxf & operator |= ( const Coord3f & p );
		// Intersezione di box
    inline  Boxf & Intersect( const Boxf & b );
	inline  Boxf & operator &= ( const Boxf & b );
    inline  Boxf & Translate( const Coord3f & p );
    inline  bool  IsIn( Coord3f const & p ) const;
    inline  int   IsEmpty( void ) const;
    inline  float Diag( void ) const;
    inline  float Volume( void ) const;
    inline  float DimX( void ) const;
    inline  float DimY( void ) const;
    inline  float DimZ( void ) const;
};


inline Boxf & Boxf::Inflate( const float s )
{
    Inflate( (max-min)*s );
	return *this;
}


inline Boxf & Boxf::Inflate( const Coord3f & delta )
{
    min -= delta;
    max += delta;
	return *this;
}


inline Boxf & Boxf::operator += ( const Coord3f & delta )
{
    min -= delta;
    max += delta;
	return *this;
}


inline Boxf & Boxf::Set( const Coord3f & p )
{
    min = max = p;
	return *this;
}


inline Boxf & Boxf::operator = ( const Coord3f & p )
{
    min = max = p;
	return *this;
}


inline Boxf & Boxf::Add( Boxf const & b )
{
    if(min.x > b.min.x) min.x = b.min.x;
    if(min.y > b.min.y) min.y = b.min.y;
    if(min.z > b.min.z) min.z = b.min.z;

    if(max.x < b.max.x) max.x = b.max.x;
    if(max.y < b.max.y) max.y = b.max.y;
    if(max.z < b.max.z) max.z = b.max.z;
	return *this;
}


inline Boxf & Boxf::operator |= ( Boxf const & b )
{
    if(min.x > b.min.x) min.x = b.min.x;
    if(min.y > b.min.y) min.y = b.min.y;
    if(min.z > b.min.z) min.z = b.min.z;

    if(max.x < b.max.x) max.x = b.max.x;
    if(max.y < b.max.y) max.y = b.max.y;
    if(max.z < b.max.z) max.z = b.max.z;
	return *this;
}


inline Boxf & Boxf::Add( const Coord3f & p )
{
    if(min.x > p.x) min.x = p.x;
    if(min.y > p.y) min.y = p.y;
    if(min.z > p.z) min.z = p.z;

    if(max.x < p.x) max.x = p.x;
    if(max.y < p.y) max.y = p.y;
    if(max.z < p.z) max.z = p.z;
	return *this;
}


inline Boxf & Boxf::operator |= ( const Coord3f & p )
{
    if(min.x > p.x) min.x = p.x;
    if(min.y > p.y) min.y = p.y;
    if(min.z > p.z) min.z = p.z;

    if(max.x < p.x) max.x = p.x;
    if(max.y < p.y) max.y = p.y;
    if(max.z < p.z) max.z = p.z;
	return *this;
}


inline Boxf & Boxf::Intersect( const Boxf & b )
{
    if(min.x < b.min.x) min.x = b.min.x;
    if(min.y < b.min.y) min.y = b.min.y;
    if(min.z < b.min.z) min.z = b.min.z;

    if(max.x > b.max.x) max.x = b.max.x;
    if(max.y > b.max.y) max.y = b.max.y;
    if(max.z > b.max.z) max.z = b.max.z;

	if(max.x < min.x) max.x = min.x = 0;
	if(max.y < min.y) max.y = min.y = 0;
	if(max.z < min.z) max.z = min.z = 0;
	
	return *this;
}


inline Boxf & Boxf::operator &= ( const Boxf & b )
{
    if(min.x < b.min.x) min.x = b.min.x;
    if(min.y < b.min.y) min.y = b.min.y;
    if(min.z < b.min.z) min.z = b.min.z;

    if(max.x > b.max.x) max.x = b.max.x;
    if(max.y > b.max.y) max.y = b.max.y;
    if(max.z > b.max.z) max.z = b.max.z;

	if(max.x < min.x) max.x = min.x = 0;
	if(max.y < min.y) max.y = min.y = 0;
	if(max.z < min.z) max.z = min.z = 0;
	
	return *this;
}


inline Boxf & Boxf::Translate( const Coord3f & p )
{
    min += p;
    max += p;
	return *this;
}


inline bool Boxf::IsIn( Coord3f const & p ) const
{
    return (
        min.x <= p.x && p.x >= max.x &&
        min.y <= p.y && p.y >= max.y &&
        min.z <= p.z && p.z >= max.z
    );
}


inline int Boxf::IsEmpty( void ) const
{
    return min==max;
}


inline float Boxf::Diag( void ) const
{
    return Dist(min,max);
}


inline  float Boxf::Volume( void ) const
{
    return (max.x-min.x)*(max.y-min.y)*(max.z-min.z);
}


inline float Boxf::DimX( void ) const
{
    return max.x-min.x;
}


inline float Boxf::DimY( void ) const
{
    return max.y-min.y;
}


inline float Boxf::DimZ( void ) const
{
    return max.z-min.z;
}


#endif