
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD2F
#define GR_SHQN_COORD2F


#include <stdio.h>
#include <math.h>

	
	/** Bidimensional point.
		Bidimensional point (float), see Coord3f for the member definition, except
		cross product.
	  */
class Coord2f
{
public:
    
    float x,y;

    inline Coord2f ( void ) { }
    inline Coord2f ( const float nx, const float ny );
    inline Coord2f ( Coord2f const & v);
    inline Coord2f & operator = ( Coord2f const & v);
    inline Coord2f & Set( const float X, const float Y );
    inline Coord2f & Get( float & X, float & Y );
    inline float & operator [] ( const int i );
    inline Coord2f operator + ( Coord2f const & v) const;
    inline Coord2f operator - ( Coord2f const & v) const;
    inline Coord2f operator * ( const float s ) const;
    inline Coord2f operator / ( const float s ) const;
    inline float Norm( void ) const;
    inline float Norm2( void ) const;
    inline float operator * ( Coord2f const & v ) const;      // Dot product
    inline Coord2f & operator += ( Coord2f const & v);
    inline Coord2f & operator -= ( Coord2f const & v);
    inline Coord2f & operator *= ( const float s );
    inline Coord2f & operator /= ( const float s );
    inline Coord2f & Scale2( const float sx, const float sy );
    inline Coord2f & Normalize( void );
    inline bool operator == ( Coord2f const & v ) const ;
    inline bool operator != ( Coord2f const & v ) const ;
	inline bool  eq ( Coord2f const & v, const float eps ) const ;
	inline bool neq ( Coord2f const & v, const float eps ) const ;
    inline bool operator <  ( Coord2f const & v ) const ;
    inline bool operator >  ( Coord2f const & v ) const ;
    inline bool operator <= ( Coord2f const & v ) const ;
    inline bool operator >= ( Coord2f const & v ) const ;
    inline int qsortcomp( Coord2f const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline float Hash( void ) const;			// range [0-1)
    inline float Dist( Coord2f const & p ) const;
};


inline Coord2f	operator - ( Coord2f const & v );
inline Coord2f	operator * ( const float s, Coord2f const & v );
inline float	Dist( Coord2f const & v1, Coord2f const & v2 );
inline Coord2f & Normalize( Coord2f & v );
inline float	Norm( Coord2f const & v );
inline float    Norm2( Coord2f const & v );


inline Coord2f::Coord2f ( const float nx, const float ny )
{
    x = nx;
    y = ny;
}


inline Coord2f::Coord2f( Coord2f const & v)
{
    x= v.x;
    y= v.y;
}


inline Coord2f & Coord2f::operator =( Coord2f const & v)
{
    x= v.x;
    y= v.y;
    return *this;
}


inline Coord2f & Coord2f::Set( const float X, const float Y )
{
    x= X;
    y= Y;
    return *this;
} 


inline Coord2f & Coord2f::Get( float & X, float & Y )
{
    X= x;
    Y= y;
    return *this;
}


inline float & Coord2f::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
    }
    return x;
}


inline Coord2f Coord2f::operator + ( Coord2f const & v) const
{
    return Coord2f( x+v.x, y+v.y );
}


inline Coord2f Coord2f::operator - ( Coord2f const & v) const
{
    return Coord2f( x-v.x, y-v.y );
}


inline Coord2f Coord2f::operator * ( const float s ) const
{
    return Coord2f( x*s, y*s );
}


inline Coord2f Coord2f::operator / ( const float s ) const
{
    return Coord2f( x / s, y / s );
} 


inline float Coord2f::operator * ( Coord2f const & v ) const
{
    return ( x*v.x + y*v.y );
}


inline float Coord2f::Norm( void ) const
{
    return sqrtf( x*x + y*y );
}


inline float Coord2f::Norm2( void ) const
{
    return ( x*x + y*y );
}


inline Coord2f & Coord2f::operator += ( Coord2f const & v)
{
    x += v.x;
    y += v.y;
    return *this;
}


inline Coord2f & Coord2f::operator -= ( Coord2f const & v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

inline Coord2f & Coord2f::operator *= ( const float s )
{
    x *= s;
    y *= s;
    return *this;
}


inline Coord2f & Coord2f::operator /= ( const float s )
{
    x /= s;
    y /= s;
    return *this;
}   


inline Coord2f & Coord2f::Scale2( const float sx, const float sy )
{
    x /= sx;
    y /= sy;
    return *this;
}


inline bool Coord2f::operator == ( Coord2f const & v ) const
{
    return x==v.x && y==v.y;
}   


inline bool Coord2f::eq( Coord2f const & v, const float eps ) const
{
    return ( fabsf(x-v.x)<eps &&
			 fabsf(y-v.y)<eps );
}   


inline bool Coord2f::operator != ( Coord2f const & v ) const
{
    return x!=v.x || y!=v.y;
}


inline bool Coord2f::neq( Coord2f const & v, const float eps ) const {
    return ( fabsf(x-v.x)>eps ||
	         fabsf(y-v.y)>eps );
}


inline bool Coord2f::operator <  ( Coord2f const & v ) const
{
    return (x!=v.x)?(x<v.x):(y<v.y);
}


inline bool Coord2f::operator >  ( Coord2f const & v ) const
{
    return (x!=v.x)?(x>v.x):(y>v.y);
}


inline bool Coord2f::operator <= ( Coord2f const & v ) const
{
    return (x!=v.x)?(x< v.x):(y< v.y);
}


inline bool Coord2f::operator >= ( Coord2f const & v ) const
{
    return (x!=v.x)?(x> v.x):(y> v.y);
}


inline int Coord2f::qsortcomp( Coord2f const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       0;
}


inline Coord2f & Coord2f::Normalize( void )
{
    float n = sqrtf(x*x + y*y);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
    }
    return *this;
}


inline int Coord2f::Read ( FILE *fp )
{
    return fscanf(fp,"%f%f",&x,&y);
}


inline int Coord2f::Write( FILE *fp ) const
{
    return fprintf(fp,"%g %g ",x,y);
}    


inline int Coord2f::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(float),1,fp);
    r &= fread(&y,sizeof(float),1,fp);
    return r;
}


inline int Coord2f::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(float),1,fp);
    r &= fwrite(&y,sizeof(float),1,fp);
    return r;
}


inline float Coord2f::Hash( void ) const
{
    float t;
    return modff( logf(x*x+3*y*y),&t );
}


inline float operator % ( Coord2f const & v1, Coord2f const & v2 )
{
    return (float) acosf( (v1*v2)/(v1.Norm()*v2.Norm()) );
}


inline Coord2f operator - ( Coord2f const & v ){
    return Coord2f( -v.x, -v.y );
}


inline Coord2f operator * ( const float s, Coord2f const & v )
{
    return Coord2f( v.x * s, v.y * s );
}


inline float Norm( Coord2f const & v )
{
    return sqrtf( v.x*v.x + v.y*v.y );
}


inline float Norm2( Coord2f const & v )
{
    return ( v.x*v.x + v.y*v.y );
}


inline Coord2f & Normalize( Coord2f & v )
{
    float n = sqrtf( v.x*v.x + v.y*v.y );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
    }
    return v;
}


inline float Dist( Coord2f const & v1,Coord2f const & v2 )
{
    return Norm(v1-v2);
}


inline float Coord2f::Dist( Coord2f const & p ) const
{
    return (*this-p).Norm();
}


#endif
