
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD2I
#define GR_SHQN_COORD2I


#include <stdio.h>


//namespace gcr
//{
	
	/** Bidimensional point.
		Bidimensional point (int), see Coord3f for the member definition, except
		cross product.
	  */
class Coord2i
{
public:
    
    int x,y;

    inline Coord2i ( void ) { }
    inline Coord2i ( const int nx, const int ny );
    inline Coord2i ( Coord2i const & v);
    inline Coord2i & operator = ( Coord2i const & v);
    inline Coord2i & Set( const int X, const int Y );
    inline Coord2i & Get( int & X, int & Y );
    inline int & operator [] ( const int i );
    inline Coord2i operator + ( Coord2i const & v) const;
    inline Coord2i operator - ( Coord2i const & v) const;
    inline Coord2i operator * ( const int s ) const;
    inline Coord2i operator / ( const int s ) const;
    inline int Norm2( void ) const;
    inline int operator * ( Coord2i const & v ) const;      // Dot product
    inline Coord2i & operator += ( Coord2i const & v);
    inline Coord2i & operator -= ( Coord2i const & v);
    inline Coord2i & operator *= ( const int s );
    inline Coord2i & operator /= ( const int s );
    inline Coord2i & Scale2( const int sx, const int sy );
    inline bool operator == ( Coord2i const & v ) const ;
    inline bool operator != ( Coord2i const & v ) const ;
    inline bool operator <  ( Coord2i const & v ) const ;
    inline bool operator >  ( Coord2i const & v ) const ;
    inline bool operator <= ( Coord2i const & v ) const ;
    inline bool operator >= ( Coord2i const & v ) const ;
    inline int qsortcomp( Coord2i const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
};


inline Coord2i	operator - ( Coord2i const & v );
inline Coord2i	operator * ( const int s, Coord2i const & v );
inline int Norm2( Coord2i const & v );


inline Coord2i::Coord2i ( const int nx, const int ny )
{
    x = nx;
    y = ny;
}


inline Coord2i::Coord2i( Coord2i const & v)
{
    x= v.x;
    y= v.y;
}


inline Coord2i & Coord2i::operator =( Coord2i const & v)
{
    x= v.x;
    y= v.y;
    return *this;
}


inline Coord2i & Coord2i::Set( const int X, const int Y )
{
    x= X;
    y= Y;
    return *this;
} 


inline Coord2i & Coord2i::Get( int & X, int & Y )
{
    X= x;
    Y= y;
    return *this;
}


inline int & Coord2i::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
    }
    return x;
}


inline Coord2i Coord2i::operator + ( Coord2i const & v) const
{
    return Coord2i( x+v.x, y+v.y );
}


inline Coord2i Coord2i::operator - ( Coord2i const & v) const
{
    return Coord2i( x-v.x, y-v.y );
}


inline Coord2i Coord2i::operator * ( const int s ) const
{
    return Coord2i( x*s, y*s );
}


inline Coord2i Coord2i::operator / ( const int s ) const
{
    return Coord2i( x / s, y / s );
} 


inline int Coord2i::operator * ( Coord2i const & v ) const
{
    return ( x*v.x + y*v.y );
}


inline int Coord2i::Norm2( void ) const
{
    return ( x*x + y*y );
}


inline Coord2i & Coord2i::operator += ( Coord2i const & v)
{
    x += v.x;
    y += v.y;
    return *this;
}


inline Coord2i & Coord2i::operator -= ( Coord2i const & v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

inline Coord2i & Coord2i::operator *= ( const int s )
{
    x *= s;
    y *= s;
    return *this;
}


inline Coord2i & Coord2i::operator /= ( const int s )
{
    x /= s;
    y /= s;
    return *this;
}   


inline Coord2i & Coord2i::Scale2( const int sx, const int sy )
{
    x /= sx;
    y /= sy;
    return *this;
}


inline bool Coord2i::operator == ( Coord2i const & v ) const
{
    return x==v.x && y==v.y;
}   


inline bool Coord2i::operator != ( Coord2i const & v ) const
{
    return x!=v.x || y!=v.y;
}


inline bool Coord2i::operator <  ( Coord2i const & v ) const
{
    return (x!=v.x)?(x<v.x):(y<v.y);
}


inline bool Coord2i::operator >  ( Coord2i const & v ) const
{
    return (x!=v.x)?(x>v.x):(y>v.y);
}


inline bool Coord2i::operator <= ( Coord2i const & v ) const
{
    return (x!=v.x)?(x< v.x):(y< v.y);
}


inline bool Coord2i::operator >= ( Coord2i const & v ) const
{
    return (x!=v.x)?(x> v.x):(y> v.y);
}


inline int Coord2i::qsortcomp( Coord2i const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       0;
}


inline int Coord2i::Read ( FILE *fp )
{
    return fscanf(fp,"%f%f",&x,&y);
}


inline int Coord2i::Write( FILE *fp ) const
{
    return fprintf(fp,"%g %g ",x,y);
}    


inline int Coord2i::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(int),1,fp);
    r &= fread(&y,sizeof(int),1,fp);
    return r;
}


inline int Coord2i::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(int),1,fp);
    r &= fwrite(&y,sizeof(int),1,fp);
    return r;
}


inline Coord2i operator - ( Coord2i const & v ){
    return Coord2i( -v.x, -v.y );
}


inline Coord2i operator * ( const int s, Coord2i const & v )
{
    return Coord2i( v.x * s, v.y * s );
}


inline int Norm2( Coord2i const & v )
{
    return ( v.x*v.x + v.y*v.y );
}


//}	// End namespace


#endif
