
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD2S
#define GR_SHQN_COORD2S


#include <stdio.h>

#pragma warning ( once : 4244 )

//namespace gcr
//{
	
	/** Bidimensional point.
		Bidimensional point (short), see Coord3f for the member definition, except
		cross product.
	  */
class Coord2s
{
public:
    
    short x,y;

    inline Coord2s ( void ) { }
    inline Coord2s ( const short nx, const short ny );
    inline Coord2s ( Coord2s const & v);
    inline Coord2s & operator = ( Coord2s const & v);
    inline Coord2s & Set( const short X, const short Y );
    inline Coord2s & Get( short & X, short & Y );
    inline short & operator [] ( const int i );
    inline Coord2s operator + ( Coord2s const & v) const;
    inline Coord2s operator - ( Coord2s const & v) const;
    inline Coord2s operator * ( const short s ) const;
    inline Coord2s operator / ( const short s ) const;
    inline short Norm2( void ) const;
    inline short operator * ( Coord2s const & v ) const;      // Dot product
    inline Coord2s & operator += ( Coord2s const & v);
    inline Coord2s & operator -= ( Coord2s const & v);
    inline Coord2s & operator *= ( const short s );
    inline Coord2s & operator /= ( const short s );
    inline Coord2s & Scale2( const short sx, const short sy );
    inline bool operator == ( Coord2s const & v ) const ;
    inline bool operator != ( Coord2s const & v ) const ;
    inline bool operator <  ( Coord2s const & v ) const ;
    inline bool operator >  ( Coord2s const & v ) const ;
    inline bool operator <= ( Coord2s const & v ) const ;
    inline bool operator >= ( Coord2s const & v ) const ;
    inline int qsortcomp( Coord2s const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
};


inline Coord2s	operator - ( Coord2s const & v );
inline Coord2s	operator * ( const short s, Coord2s const & v );
inline short Norm2( Coord2s const & v );


inline Coord2s::Coord2s ( const short nx, const short ny )
{
    x = nx;
    y = ny;
}


inline Coord2s::Coord2s( Coord2s const & v)
{
    x= v.x;
    y= v.y;
}


inline Coord2s & Coord2s::operator =( Coord2s const & v)
{
    x= v.x;
    y= v.y;
    return *this;
}


inline Coord2s & Coord2s::Set( const short X, const short Y )
{
    x= X;
    y= Y;
    return *this;
} 


inline Coord2s & Coord2s::Get( short & X, short & Y )
{
    X= x;
    Y= y;
    return *this;
}


inline short & Coord2s::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
    }
    return x;
}


inline Coord2s Coord2s::operator + ( Coord2s const & v) const
{
    return Coord2s( x+v.x, y+v.y );
}


inline Coord2s Coord2s::operator - ( Coord2s const & v) const
{
    return Coord2s( x-v.x, y-v.y );
}


inline Coord2s Coord2s::operator * ( const short s ) const
{
    return Coord2s( x*s, y*s );
}


inline Coord2s Coord2s::operator / ( const short s ) const
{
    return Coord2s( x / s, y / s );
} 


inline short Coord2s::operator * ( Coord2s const & v ) const
{
    return ( x*v.x + y*v.y );
}


inline short Coord2s::Norm2( void ) const
{
    return ( x*x + y*y );
}


inline Coord2s & Coord2s::operator += ( Coord2s const & v)
{
    x += v.x;
    y += v.y;
    return *this;
}


inline Coord2s & Coord2s::operator -= ( Coord2s const & v)
{
    x -= v.x;
    y -= v.y;
    return *this;
}

inline Coord2s & Coord2s::operator *= ( const short s )
{
    x *= s;
    y *= s;
    return *this;
}


inline Coord2s & Coord2s::operator /= ( const short s )
{
    x /= s;
    y /= s;
    return *this;
}   


inline Coord2s & Coord2s::Scale2( const short sx, const short sy )
{
    x /= sx;
    y /= sy;
    return *this;
}


inline bool Coord2s::operator == ( Coord2s const & v ) const
{
    return x==v.x && y==v.y;
}   


inline bool Coord2s::operator != ( Coord2s const & v ) const
{
    return x!=v.x || y!=v.y;
}


inline bool Coord2s::operator <  ( Coord2s const & v ) const
{
    return (x!=v.x)?(x<v.x):(y<v.y);
}


inline bool Coord2s::operator >  ( Coord2s const & v ) const
{
    return (x!=v.x)?(x>v.x):(y>v.y);
}


inline bool Coord2s::operator <= ( Coord2s const & v ) const
{
    return (x!=v.x)?(x< v.x):(y< v.y);
}


inline bool Coord2s::operator >= ( Coord2s const & v ) const
{
    return (x!=v.x)?(x> v.x):(y> v.y);
}


inline int Coord2s::qsortcomp( Coord2s const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       0;
}


inline int Coord2s::Read ( FILE *fp )
{
    return fscanf(fp,"%f%f",&x,&y);
}


inline int Coord2s::Write( FILE *fp ) const
{
    return fprintf(fp,"%g %g ",x,y);
}    


inline int Coord2s::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(short),1,fp);
    r &= fread(&y,sizeof(short),1,fp);
    return r;
}


inline int Coord2s::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(short),1,fp);
    r &= fwrite(&y,sizeof(short),1,fp);
    return r;
}


inline Coord2s operator - ( Coord2s const & v ){
    return Coord2s( -v.x, -v.y );
}


inline Coord2s operator * ( const short s, Coord2s const & v )
{
    return Coord2s( v.x * s, v.y * s );
}


inline short Norm2( Coord2s const & v )
{
    return ( v.x*v.x + v.y*v.y );
}


//}	// End namespace

#pragma warning ( default : 4244 )

#endif
