
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD3I
#define GR_SHQN_COORD3I


#include <stdio.h>


//namespace gcr
//{
	
	
	/** Tridimensional point.
		Tridimensional point (int), see Coord3f for the member definition.
	  */
class Coord3i
{
public:
    
    int x,y,z;

    inline Coord3i ( void ) { }
    inline Coord3i ( const int nx, const int ny, const int nz );
    inline Coord3i ( Coord3i const & v);
    inline Coord3i & operator = ( Coord3i const & v);
    inline Coord3i & Set( const int X, const int Y, const int Z );
    inline Coord3i & Get( int & X, int & Y, int & Z );
    inline int & operator [] ( const int i );
    inline Coord3i operator + ( Coord3i const & v) const;
    inline Coord3i operator - ( Coord3i const & v) const;
    inline Coord3i operator * ( const int s ) const;
    inline Coord3i operator / ( const int s ) const;
    inline int Norm2( void ) const;
    inline int operator * ( Coord3i const & v ) const;      // Dot product
    inline Coord3i operator ^ ( Coord3i const & v ) const;   // Cross Product
    inline Coord3i & operator += ( Coord3i const & v);
    inline Coord3i & operator -= ( Coord3i const & v);
    inline Coord3i & operator *= ( const int s );
    inline Coord3i & operator /= ( const int s );
    inline Coord3i & Scale3( const int sx, const int sy, const int sz );
    inline bool operator == ( Coord3i const & v ) const ;
    inline bool operator != ( Coord3i const & v ) const ;
    inline bool operator <  ( Coord3i const & v ) const ;
    inline bool operator >  ( Coord3i const & v ) const ;
    inline bool operator <= ( Coord3i const & v ) const ;
    inline bool operator >= ( Coord3i const & v ) const ;
    inline int qsortcomp( Coord3i const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
};


inline Coord3i	operator - ( Coord3i const & v );
inline Coord3i	operator * ( const int s, Coord3i const & v );
inline int Norm2( Coord3i const & v );


inline Coord3i::Coord3i ( const int nx, const int ny, const int nz )
{
    x = nx;
    y = ny;
    z = nz;
}


inline Coord3i::Coord3i( Coord3i const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
}


inline Coord3i & Coord3i::operator =( Coord3i const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
    return *this;
}


inline Coord3i & Coord3i::Set( const int X, const int Y, const int Z )
{
    x= X;
    y= Y;
    z= Z;
    return *this;
} 


inline Coord3i & Coord3i::Get( int & X, int & Y, int & Z )
{
    X= x;
    Y= y;
    Z= z;
    return *this;
}


inline int & Coord3i::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
    }
    return x;
}


inline Coord3i Coord3i::operator + ( Coord3i const & v) const
{
    return Coord3i( x+v.x, y+v.y, z+v.z );
}


inline Coord3i Coord3i::operator - ( Coord3i const & v) const
{
    return Coord3i( x-v.x, y-v.y, z-v.z );
}


inline Coord3i Coord3i::operator * ( const int s ) const
{
    return Coord3i( x*s, y*s, z*s );
}


inline Coord3i Coord3i::operator / ( const int s ) const
{
    return Coord3i( x / s, y / s, z / s );
} 


inline int Coord3i::operator * ( Coord3i const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z );
}


inline Coord3i Coord3i::operator ^ ( Coord3i const & v ) const
{
    return Coord3i(
        y*v.z - z*v.y,
        z*v.x - x*v.z,
        x*v.y - y*v.x
    );
}


inline int Coord3i::Norm2( void ) const
{
    return x*x + y*y + z*z;
}


inline Coord3i & Coord3i::operator += ( Coord3i const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
}


inline Coord3i & Coord3i::operator -= ( Coord3i const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
    return *this;
}

inline Coord3i & Coord3i::operator *= ( const int s )
{
    x *= s;
    y *= s;
    z *= s;
    return *this;
}


inline Coord3i & Coord3i::operator /= ( const int s )
{
    x /= s;
    y /= s;
    z /= s;
    return *this;
}   


inline Coord3i & Coord3i::Scale3( const int sx, const int sy, const int sz )
{
    x /= sx;
    y /= sy;
    z /= sz;
    return *this;
}


inline bool Coord3i::operator == ( Coord3i const & v ) const
{
    return x==v.x && y==v.y && z==v.z;
}   


inline bool Coord3i::operator != ( Coord3i const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z;
}


inline bool Coord3i::operator <  ( Coord3i const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z<v.z);
}


inline bool Coord3i::operator >  ( Coord3i const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z>v.z);
}


inline bool Coord3i::operator <= ( Coord3i const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z<=v.z);
}


inline bool Coord3i::operator >= ( Coord3i const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z>=v.z);
}


inline int Coord3i::qsortcomp( Coord3i const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
	       0;
}


inline int Coord3i::Read ( FILE *fp )
{
    return fscanf(fp,"%d%d%d",&x,&y,&z);
}


inline int Coord3i::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%d %d %d ",x,y,z));
}    


inline int Coord3i::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(int),1,fp);
    r &= fread(&y,sizeof(int),1,fp);
    r &= fread(&z,sizeof(int),1,fp);
    return r;
}


inline int Coord3i::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(int),1,fp);
    r &= fwrite(&y,sizeof(int),1,fp);
    r &= fwrite(&z,sizeof(int),1,fp);
    return r;
}


inline Coord3i operator - ( Coord3i const & v ){
    return Coord3i( -v.x, -v.y, -v.z );
}


inline Coord3i operator * ( const int s, Coord3i const & v )
{
    return Coord3i( v.x * s, v.y * s, v.z * s );
}


inline int Norm2( Coord3i const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z );
}


//}	// End namespace


#endif
