
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD3S
#define GR_SHQN_COORD3S


#include <stdio.h>

#pragma warning ( once : 4244 )

//namespace gcr
//{
	
	/** Tridimensional point.
		Tridimensional point (short), see Coord3f for the member definition.
	  */
class Coord3s
{
public:
    
    short x,y,z;

    inline Coord3s ( void ) { }
    inline Coord3s ( const short nx, const short ny, const short nz );
	inline Coord3s ( const int nx, const int ny, const int nz );
    inline Coord3s ( Coord3s const & v);
    inline Coord3s & operator = ( Coord3s const & v);
    inline Coord3s & Set( const short X, const short Y, const short Z );
    inline Coord3s & Get( short & X, short & Y, short & Z );
    inline short & operator [] ( const short i );
    inline Coord3s operator + ( Coord3s const & v) const;
    inline Coord3s operator - ( Coord3s const & v) const;
    inline Coord3s operator * ( const short s ) const;
    inline Coord3s operator / ( const short s ) const;
    inline short Norm2( void ) const;
    inline short operator * ( Coord3s const & v ) const;      // Dot product
    inline Coord3s operator ^ ( Coord3s const & v ) const;   // Cross Product
    inline Coord3s & operator += ( Coord3s const & v);
    inline Coord3s & operator -= ( Coord3s const & v);
    inline Coord3s & operator *= ( const short s );
    inline Coord3s & operator /= ( const short s );
    inline Coord3s & Scale3( const short sx, const short sy, const short sz );
    inline bool operator == ( Coord3s const & v ) const ;
	inline bool operator != ( Coord3s const & v ) const ;
    inline bool operator <  ( Coord3s const & v ) const ;
    inline bool operator >  ( Coord3s const & v ) const ;
    inline bool operator <= ( Coord3s const & v ) const ;
    inline bool operator >= ( Coord3s const & v ) const ;
    inline int qsortcomp( Coord3s const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
};


inline Coord3s	operator - ( Coord3s const & v );
inline Coord3s	operator * ( const short s, Coord3s const & v );
inline short Norm2( Coord3s const & v );
inline short Norm2( Coord3s const & v );


inline Coord3s::Coord3s ( const short nx, const short ny, const short nz )
{
    x = nx;
    y = ny;
    z = nz;
}


inline Coord3s::Coord3s ( const int nx, const int ny, const int nz )
{
    x = short(nx);
    y = short(ny);
    z = short(nz);
}

inline Coord3s::Coord3s( Coord3s const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
}


inline Coord3s & Coord3s::operator =( Coord3s const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
    return *this;
}


inline Coord3s & Coord3s::Set( const short X, const short Y, const short Z )
{
    x= X;
    y= Y;
    z= Z;
    return *this;
} 


inline Coord3s & Coord3s::Get( short & X, short & Y, short & Z )
{
    X= x;
    Y= y;
    Z= z;
    return *this;
}


inline short & Coord3s::operator [] ( const short i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
    }
    return x;
}


inline Coord3s Coord3s::operator + ( Coord3s const & v) const
{
    return Coord3s( x+v.x, y+v.y, z+v.z );
}


inline Coord3s Coord3s::operator - ( Coord3s const & v) const
{
    return Coord3s( x-v.x, y-v.y, z-v.z );
}


inline Coord3s Coord3s::operator * ( const short s ) const
{
    return Coord3s( x*s, y*s, z*s );
}


inline Coord3s Coord3s::operator / ( const short s ) const
{
    return Coord3s( x / s, y / s, z / s );
} 


inline short Coord3s::operator * ( Coord3s const & v ) const
{
    return short( x*v.x + y*v.y + z*v.z );
}


inline Coord3s Coord3s::operator ^ ( Coord3s const & v ) const
{
    return Coord3s(
        y*v.z - z*v.y,
        z*v.x - x*v.z,
        x*v.y - y*v.x
    );
}


inline short Coord3s::Norm2( void ) const
{
    return short(x*x + y*y + z*z);
}


inline Coord3s & Coord3s::operator += ( Coord3s const & v)
{
    x = short(x+v.x);
    y = short(y+v.y);
    z = short(z+v.z);
    return *this;
}


inline Coord3s & Coord3s::operator -= ( Coord3s const & v)
{
    x = short(x-v.x);
    y = short(y-v.y);
    z = short(z-v.z);
    return *this;
}

inline Coord3s & Coord3s::operator *= ( const short s )
{
    x = short(x*s);
    y = short(y*s);
    z = short(z*s);
    return *this;
}


inline Coord3s & Coord3s::operator /= ( const short s )
{
    x = short(x/s);
    y = short(y/s);
    z = short(z/s);
    return *this;
}   


inline Coord3s & Coord3s::Scale3( const short sx, const short sy, const short sz )
{
    x = short(x/sx);
    y = short(y/sy);
    z = short(z/sz);
    return *this;
}


inline bool Coord3s::operator == ( Coord3s const & v ) const
{
    return x==v.x && y==v.y && z==v.z;
}   


inline bool Coord3s::operator != ( Coord3s const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z;
}


inline bool Coord3s::operator <  ( Coord3s const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z<v.z);
}


inline bool Coord3s::operator >  ( Coord3s const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z>v.z);
}


inline bool Coord3s::operator <= ( Coord3s const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z<=v.z);
}


inline bool Coord3s::operator >= ( Coord3s const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z>=v.z);
}


inline int Coord3s::qsortcomp( Coord3s const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
	       0;
}


inline int Coord3s::Read ( FILE *fp )
{
	int r,nx,ny,nz;
    r = fscanf(fp,"%d%d%d",&nx,&ny,&nz);
	x = short(nx);
	y = short(ny);
	z = short(nz);
	return r;
}


inline int Coord3s::Write( FILE *fp ) const
{
    return fprintf(fp,"%d %d %d ",x,y,z);
}    


inline int Coord3s::ReadBin( FILE *fp ) {
    int r = 1;
    r &= fread(&x,sizeof(short),1,fp);
    r &= fread(&y,sizeof(short),1,fp);
    r &= fread(&z,sizeof(short),1,fp);
    return r;
}


inline int Coord3s::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(short),1,fp);
    r &= fwrite(&y,sizeof(short),1,fp);
    r &= fwrite(&z,sizeof(short),1,fp);
    return r;
}


inline Coord3s operator - ( Coord3s const & v ){
    return Coord3s( -v.x, -v.y, -v.z );
}


inline Coord3s operator * ( const short s, Coord3s const & v )
{
    return Coord3s( v.x * s, v.y * s, v.z * s );
}


inline short Norm2( Coord3s const & v )
{
    return short( v.x*v.x + v.y*v.y + v.z*v.z );
}

#pragma warning ( default : 4244 )

//}	// End namespace


#endif
