
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD4D
#define GR_SHQN_COORD4D


#include <stdio.h>
#include <math.h>


//namespace gcr
//{
	
	/** Quadrimensional point.
		Quadrimensional point (double), see Coord3f for the member definition, except
		cross product.
	  */
class Coord4d
{
public:
    
    double x,y,z,w;

    inline Coord4d ( void ) { }
    inline Coord4d ( const double nx, const double ny,
		             const double nz, const double nw );
    inline Coord4d ( Coord4d const & v);
    inline Coord4d & operator = ( Coord4d const & v);
    inline Coord4d & Set( const double X, const double Y, const double Z, const double W );
    inline Coord4d & Get( double & X, double & Y, double & Z, double & W );
    inline double & operator [] ( const int i );
    inline Coord4d operator + ( Coord4d const & v) const;
    inline Coord4d operator - ( Coord4d const & v) const;
    inline Coord4d operator * ( const double s ) const;
    inline Coord4d operator / ( const double s ) const;
    inline double Norm( void ) const;
    inline double Norm2( void ) const;
    inline double operator * ( Coord4d const & v ) const;      // Dot product
    inline Coord4d & operator += ( Coord4d const & v);
    inline Coord4d & operator -= ( Coord4d const & v);
    inline Coord4d & operator *= ( const double s );
    inline Coord4d & operator /= ( const double s );
    inline Coord4d & Scale4( const double sx, const double sy,
		                     const double sz, const double sw );
    inline Coord4d & Normalize( void );
    inline bool operator == ( Coord4d const & v ) const ;
    inline bool operator != ( Coord4d const & v ) const ;
	inline bool  eq ( Coord4d const & v, const double eps ) const ;
	inline bool neq ( Coord4d const & v, const double eps ) const ;
    inline bool operator <  ( Coord4d const & v ) const ;
    inline bool operator >  ( Coord4d const & v ) const ;
    inline bool operator <= ( Coord4d const & v ) const ;
    inline bool operator >= ( Coord4d const & v ) const ;
    inline int qsortcomp( Coord4d const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
    inline double Hash( void ) const;			// range [0-1)
    inline double Dist( Coord4d const & p ) const;
};


inline Coord4d	operator - ( Coord4d const & v );
inline Coord4d	operator * ( const double s, Coord4d const & v );
inline double	Dist( Coord4d const & v1, Coord4d const & v2 );
inline Coord4d & Normalize( Coord4d & v );
inline double	Norm( Coord4d const & v );
inline double   Norm2( Coord4d const & v );


inline Coord4d::Coord4d ( const double nx, const double ny,
						  const double nz, const double nw )
{
    x = nx;
    y = ny;
    z = nz;
	w = nw;
}


inline Coord4d::Coord4d( Coord4d const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
}


inline Coord4d & Coord4d::operator =( Coord4d const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
    return *this;
}


inline Coord4d & Coord4d::Set( const double X, const double Y,
							   const double Z, const double W )
{
    x= X;
    y= Y;
    z= Z;
	w= W;
    return *this;
} 


inline Coord4d & Coord4d::Get( double & X, double & Y, double & Z, double & W )
{
    X= x;
    Y= y;
    Z= z;
	W= w;
    return *this;
}


inline double & Coord4d::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
		case 3 : return w;
    }
    return x;
}


inline Coord4d Coord4d::operator + ( Coord4d const & v) const
{
    return Coord4d( x+v.x, y+v.y, z+v.z, w+v.w );
}


inline Coord4d Coord4d::operator - ( Coord4d const & v) const
{
    return Coord4d( x-v.x, y-v.y, z-v.z, w-v.w );
}


inline Coord4d Coord4d::operator * ( const double s ) const
{
    return Coord4d( x*s, y*s, z*s, w*s );
}


inline Coord4d Coord4d::operator / ( const double s ) const
{
    return Coord4d( x / s, y / s, z / s, w / s );
} 


inline double Coord4d::operator * ( Coord4d const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z +w*v.w );
}


inline double Coord4d::Norm( void ) const
{
    return sqrt( x*x + y*y + z*z + w*w );
}


inline double Coord4d::Norm2( void ) const
{
    return ( x*x + y*y + z*z *w*w );
}


inline Coord4d & Coord4d::operator += ( Coord4d const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
	w += v.w;
    return *this;
}


inline Coord4d & Coord4d::operator -= ( Coord4d const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
	w -= v.w;
    return *this;
}

inline Coord4d & Coord4d::operator *= ( const double s )
{
    x *= s;
    y *= s;
    z *= s;
	w *= s;
    return *this;
}


inline Coord4d & Coord4d::operator /= ( const double s )
{
    x /= s;
    y /= s;
    z /= s;
	w /= s;
    return *this;
}   


inline Coord4d & Coord4d::Scale4( const double sx, const double sy,
								  const double sz, const double sw )
{
    x /= sx;
    y /= sy;
    z /= sz;
	w /= sw;
    return *this;
}


inline bool Coord4d::operator == ( Coord4d const & v ) const
{
    return x==v.x && y==v.y && z==v.z && w==v.w;
}   


inline bool Coord4d::eq( Coord4d const & v, const double eps ) const
{
    return ( fabs(x-v.x)<eps &&
			 fabs(y-v.y)<eps &&
	         fabs(z-v.z)<eps &&
			 fabs(w-v.w)<eps );
}   


inline bool Coord4d::operator != ( Coord4d const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z || w!=v.w;
}


inline bool Coord4d::neq( Coord4d const & v, const double eps ) const {
    return ( fabs(x-v.x)>eps ||
	         fabs(y-v.y)>eps ||
	         fabs(z-v.z)>eps ||
			 fabs(w-v.w)>eps  );
}


inline bool Coord4d::operator <  ( Coord4d const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z!=v.z)?(z<v.z):
		            (w<v.w);
}


inline bool Coord4d::operator >  ( Coord4d const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z!=v.z)?(z>v.z):
		            (w>v.w);
}


inline bool Coord4d::operator <= ( Coord4d const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z!=v.z)?(z< v.z):
					(w<=v.w);
}


inline bool Coord4d::operator >= ( Coord4d const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z!=v.z)?(z> v.z):
					(w>=v.w);
}


inline int Coord4d::qsortcomp( Coord4d const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
		   (w!=v.w)?( (w>v.w)?1:-1 ):
	       0;
}


inline Coord4d & Coord4d::Normalize( void )
{
    double n = sqrt(x*x + y*y + z*z + w*w);
	if(n!=0.0)
	{
		x /= n;
		y /= n;
		z /= n;
		w /= n;
    }
    return *this;
}


inline int Coord4d::Read ( FILE *fp )
{
	float nx,ny,nz,nw;
    int r = fscanf(fp,"%f%f%f%f",&nx,&ny,&nz,&nw);
    x = double(nx);
    y = double(ny);
    z = double(nz);
	w = double(nw);
	return r;
}


inline int Coord4d::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%g %g %g %g ",x,y,z,w));
}    


inline int Coord4d::ReadBin( FILE *fp ) {
    int r = 1;
    
    r &= fread(&x,sizeof(double),1,fp);
    r &= fread(&y,sizeof(double),1,fp);
    r &= fread(&z,sizeof(double),1,fp);
	r &= fread(&w,sizeof(double),1,fp);
    return r;
}


inline int Coord4d::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(double),1,fp);
    r &= fwrite(&y,sizeof(double),1,fp);
    r &= fwrite(&z,sizeof(double),1,fp);
	r &= fwrite(&w,sizeof(double),1,fp);
    return r;
}


inline double Coord4d::Hash( void ) const
{
    double t;
    return modf( log(x*x+3*y*y+5*z*z+7*w*w),&t );
}


inline Coord4d operator - ( Coord4d const & v ){
    return Coord4d( -v.x, -v.y, -v.z, -v.w );
}


inline Coord4d operator * ( const double s, Coord4d const & v )
{
    return Coord4d( v.x * s, v.y * s, v.z * s, v.w * s );
}


inline double Norm( Coord4d const & v )
{
    return sqrt( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w);
}


inline double Norm2( Coord4d const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
}


inline Coord4d & Normalize( Coord4d & v )
{
    double n = sqrt( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
    if(n>0.0){
        v.x /= n;
        v.y /= n;
        v.z /= n;
    }
    return v;
}


inline double Dist( Coord4d const & v1,Coord4d const & v2 )
{
    return Norm(v1-v2);
}


inline double Coord4d::Dist( Coord4d const & p ) const
{
    return (*this-p).Norm();
}


//}	// End namespace


#endif
