
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

        
#ifndef GR_SHQN_COORD4S
#define GR_SHQN_COORD4S


#include <stdio.h>

#pragma warning ( once : 4244 )

//namespace gcr
//{
	
	/** Quadrimensional point.
		Quadrimensional point (short), see Coord3f for the member definition, except
		cross product.
	  */
class Coord4s
{
public:
    
    short x,y,z,w;

    inline Coord4s ( void ) { }
    inline Coord4s ( const short nx, const short ny,
		             const short nz, const short nw );
    inline Coord4s ( Coord4s const & v);
    inline Coord4s & operator = ( Coord4s const & v);
    inline Coord4s & Set( const short X, const short Y, const short Z, const short W );
    inline Coord4s & Get( short & X, short & Y, short & Z, short & W );
    inline short & operator [] ( const int i );
    inline Coord4s operator + ( Coord4s const & v) const;
    inline Coord4s operator - ( Coord4s const & v) const;
    inline Coord4s operator * ( const short s ) const;
    inline Coord4s operator / ( const short s ) const;
    inline short Norm2( void ) const;
    inline short operator * ( Coord4s const & v ) const;      // Dot product
    inline Coord4s & operator += ( Coord4s const & v);
    inline Coord4s & operator -= ( Coord4s const & v);
    inline Coord4s & operator *= ( const short s );
    inline Coord4s & operator /= ( const short s );
    inline Coord4s & Scale4( const short sx, const short sy,
		                     const short sz, const short sw );
    inline bool operator == ( Coord4s const & v ) const ;
    inline bool operator != ( Coord4s const & v ) const ;
    inline bool operator <  ( Coord4s const & v ) const ;
    inline bool operator >  ( Coord4s const & v ) const ;
    inline bool operator <= ( Coord4s const & v ) const ;
    inline bool operator >= ( Coord4s const & v ) const ;
    inline int qsortcomp( Coord4s const & v ) const;
    inline int Read ( FILE *fp ) ;
    inline int Write( FILE *fp ) const;
    inline int ReadBin( FILE *fp ) ;
    inline int WriteBin( FILE *fp ) const;
};


inline Coord4s	operator - ( Coord4s const & v );
inline Coord4s	operator * ( const short s, Coord4s const & v );
inline short Norm2( Coord4s const & v );


inline Coord4s::Coord4s ( const short nx, const short ny,
						  const short nz, const short nw )
{
    x = nx;
    y = ny;
    z = nz;
	w = nw;
}


inline Coord4s::Coord4s( Coord4s const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
}


inline Coord4s & Coord4s::operator =( Coord4s const & v)
{
    x= v.x;
    y= v.y;
    z= v.z;
	w= v.w;
    return *this;
}


inline Coord4s & Coord4s::Set( const short X, const short Y,
							   const short Z, const short W )
{
    x= X;
    y= Y;
    z= Z;
	w= W;
    return *this;
} 


inline Coord4s & Coord4s::Get( short & X, short & Y, short & Z, short & W )
{
    X= x;
    Y= y;
    Z= z;
	W= w;
    return *this;
}


inline short & Coord4s::operator [] ( const int i )
{
    switch(i)
	{
        case 0 : return x;
		case 1 : return y;
		case 2 : return z;
		case 3 : return w;
    }
    return x;
}


inline Coord4s Coord4s::operator + ( Coord4s const & v) const
{
    return Coord4s( x+v.x, y+v.y, z+v.z, w+v.w );
}


inline Coord4s Coord4s::operator - ( Coord4s const & v) const
{
    return Coord4s( x-v.x, y-v.y, z-v.z, w-v.w );
}


inline Coord4s Coord4s::operator * ( const short s ) const
{
    return Coord4s( x*s, y*s, z*s, w*s );
}


inline Coord4s Coord4s::operator / ( const short s ) const
{
    return Coord4s( x / s, y / s, z / s, w / s );
} 


inline short Coord4s::operator * ( Coord4s const & v ) const
{
    return ( x*v.x + y*v.y + z*v.z +w*v.w );
}


inline short Coord4s::Norm2( void ) const
{
    return ( x*x + y*y + z*z *w*w );
}


inline Coord4s & Coord4s::operator += ( Coord4s const & v)
{
    x += v.x;
    y += v.y;
    z += v.z;
	w += v.w;
    return *this;
}


inline Coord4s & Coord4s::operator -= ( Coord4s const & v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
	w -= v.w;
    return *this;
}

inline Coord4s & Coord4s::operator *= ( const short s )
{
    x *= s;
    y *= s;
    z *= s;
	w *= s;
    return *this;
}


inline Coord4s & Coord4s::operator /= ( const short s )
{
    x /= s;
    y /= s;
    z /= s;
	w /= s;
    return *this;
}   


inline Coord4s & Coord4s::Scale4( const short sx, const short sy,
								  const short sz, const short sw )
{
    x /= sx;
    y /= sy;
    z /= sz;
	w /= sw;
    return *this;
}


inline bool Coord4s::operator == ( Coord4s const & v ) const
{
    return x==v.x && y==v.y && z==v.z && w==v.w;
}   


inline bool Coord4s::operator != ( Coord4s const & v ) const
{
    return x!=v.x || y!=v.y || z!=v.z || w!=v.w;
}


inline bool Coord4s::operator <  ( Coord4s const & v ) const
{
    return (x!=v.x)?(x<v.x):
	       (y!=v.y)?(y<v.y):
		   (z!=v.z)?(z<v.z):
		            (w<v.w);
}


inline bool Coord4s::operator >  ( Coord4s const & v ) const
{
    return (x!=v.x)?(x>v.x):
	       (y!=v.y)?(y>v.y):
		   (z!=v.z)?(z>v.z):
		            (w>v.w);
}


inline bool Coord4s::operator <= ( Coord4s const & v ) const
{
    return (x!=v.x)?(x< v.x):
	       (y!=v.y)?(y< v.y):
		   (z!=v.z)?(z< v.z):
					(w<=v.w);
}


inline bool Coord4s::operator >= ( Coord4s const & v ) const
{
    return (x!=v.x)?(x> v.x):
	       (y!=v.y)?(y> v.y):
		   (z!=v.z)?(z> v.z):
					(w>=v.w);
}


inline int Coord4s::qsortcomp( Coord4s const & v ) const
{
    return (x!=v.x)?( (x>v.x)?1:-1 ):
	       (y!=v.y)?( (y>v.y)?1:-1 ):
	       (z!=v.z)?( (z>v.z)?1:-1 ):
		   (w!=v.w)?( (w>v.w)?1:-1 ):
	       0;
}


inline int Coord4s::Read ( FILE *fp )
{
	int r,nx,ny,nz,nw;
    r = fscanf(fp,"%d%d%d%d",&nx,&ny,&nz,&nw);
	x = short(nx);
	y = short(ny);
	z = short(nz);
	w = short(nw);
	return r;
}


inline int Coord4s::Write( FILE *fp ) const
{
    return ( fprintf(fp,"%d %d %d %d ",x,y,z,w));
}    


inline int Coord4s::ReadBin( FILE *fp ) {
    int r = 1;
    
    r &= fread(&x,sizeof(short),1,fp);
    r &= fread(&y,sizeof(short),1,fp);
    r &= fread(&z,sizeof(short),1,fp);
	r &= fread(&w,sizeof(short),1,fp);
    return r;
}


inline int Coord4s::WriteBin( FILE *fp ) const
{
    int r = 1;
    r &= fwrite(&x,sizeof(short),1,fp);
    r &= fwrite(&y,sizeof(short),1,fp);
    r &= fwrite(&z,sizeof(short),1,fp);
	r &= fwrite(&w,sizeof(short),1,fp);
    return r;
}


inline Coord4s operator - ( Coord4s const & v ){
    return Coord4s( -v.x, -v.y, -v.z, -v.w );
}


inline Coord4s operator * ( const short s, Coord4s const & v )
{
    return Coord4s( v.x * s, v.y * s, v.z * s, v.w * s );
}


inline short Norm2( Coord4s const & v )
{
    return ( v.x*v.x + v.y*v.y + v.z*v.z + v.w*v.w );
}

#pragma warning ( default : 4244 )

//}	// End namespace


#endif
