
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_CYLINDER
#define GR_SHQN_CYLINDER


#include "quadric.h"

	/** Cylinder shape node.
	    This node draws a cylinder oriented along the z-axis.
		The base of the cylinder is placed at z = 0, and the top at
		z = height.
		Like a sphere, a cylinder is subdivided around the z-axis into slices,
		and along the z-axis into stacks. 
		@see GLU
	 */
class grCylinder : public grQuadric
{
public:
		/// Default constructor
	inline  grCylinder();
		/// Generate opengl calls for drawing
	inline void GL() const;
		/// see grNode
	const char * Name( void ) const;
		/// see grNode
	void Render( grStatus & status );
		/// see grNode
	void Select( grStatus & status );

		/// The radius of the cylinder at z = 0
	double baseRadius;
		/// The radius of the cylinder at z = height
	double topRadius;
		/// The height of the cylinder
	double height;
		/// The number of subdivisions around the z-axis
	int    slices;
		/// The number of subdivisions along the z-axis
	int    stacks;

private:
	static const char * name;
};


inline grCylinder::grCylinder()
{
	baseRadius = 1.0;
	topRadius  = 1.0;
	height	   = 1.0;
	slices = 16;
	stacks = 16;
}


inline void grCylinder::GL() const
{
	gluCylinder( q, baseRadius, topRadius, height, slices, stacks );
}



#endif