
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_DEPTH
#define GR_SHQN_DEPTH


#include "base.h"
#include "node.h"

	/** Z-Buffer specification node.
	    This node manipulates the z-buffer behaviour.
	 */
class grDepth : public grNode
{
public:
		/// Default constructor
	inline grDepth();
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	
		/// Buffer compare function, values: GL_NEVER, GL_LESS, GL_EQUAL, GL_LEQUAL, GL_GREATER, GL_NOTEQUAL,GL_GEQUAL,GL_ALWAYS
	GLenum func;
		/// Enables or disables writing into the depth buffer
	GLboolean writing;
		/// Z-Buffer depth range start: min value = -1
	GLclampd znear;
		/// Z-Buffer depth range end: max vaule = 1
	GLclampd zfar;

private:
	static const char * name;
};


inline grDepth::grDepth()
{
	func    = GL_LESS;
	writing = true;
	znear	= 0.0;
	zfar    = 1.0;
}


inline void grDepth::GL() const
{
	glDepthFunc(func);
	glDepthMask(writing);
	glDepthRange( znear,zfar );
}


inline void grDepth::Get()
{
	glGet(GL_DEPTH_FUNC,(int *)&func);
	glGet(GL_DEPTH_WRITEMASK,&writing);
	GLclampd tmp[2];
	glGet(GL_DEPTH_RANGE,tmp);
	znear = tmp[0];
	zfar  = tmp[1];
}


#endif