
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_GCRWIN32
#define GR_SHQN_GCRWIN32

	/** @name Win32 Interface.
		@memo Win32 intercomunication interface
	    @see win32, opengl
	 */

	//@{

const int    grZBufferBits = 32;
const double grZBufferMax  = 1<<grZBufferBits;

	/// Win32 communication data
struct grWindExtra
{
		/// Window device context
	HDC       ghDC;
		/// Window GL context
	HGLRC     ghRC;
		/// Window palette if necessary
	HPALETTE  hPale;
		/// Original Window palette
	HPALETTE  hOldP;
		/// Window is captured
	BOOL	  bCapture;
		/// Window Aspect ratio
	GLfloat	  aspect;
		/// Window Client dimension (width)
	GLint     nWidth;
		/// Window Client dimension (eight)
	GLint	  nHeight;
	
		/// Z start cutting plane 
	GLdouble  nearPlane;
		/// Z end   cutting plane
	GLdouble  farPlane;
		/// Camera Focal lenght
	GLdouble  prospAngle;

		/// Current Viewport 
	GLint	  viewport[4];
		/// Current Projection matrix
	GLdouble  projMatrix[16];
};


	/// Show a dialog error box with the message msg
void grShowError( const char * msg );

	/// Return the opengl identification string
void grGetGLInfo( char * str );

	/// Callback for window creation
BOOL grOnCreate( HWND hwnd, grWindExtra & we );

	/// Callback for window destruction
void grOnDestroy( HWND hwnd, grWindExtra & we );

	/// Callback for window resizing
void grOnSize( HWND hwnd, grWindExtra & we, int w, int h );

	//@}

#endif