

/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_HINTS
#define GR_SHQN_HINTS


#include "base.h"
#include "node.h"

	/** Implementation-specific hints.
		This node specifies implementation-specific hints of opengl rendering.
		All the setting accepts the following values:
		GL_FASTEST, GL_NICEST or GL_DONT_CARE.
		@see opengl.
	 */
	
class grHints : public grNode
{
public:
		/// Default constructor
	inline grHints();
	inline void GL() const;
		/// Retrieves settings from internal opengl status
	inline void Get();

	void Render( grStatus & status );
		/// see grNode.
	const char * Name() const;

		/// Fog hint.
	GLenum Fog;
		/// Line smoothing hint.
	GLenum LineSmoot;
		/// Perpesctive correction hint.
	GLenum PerspCorrection;
		/// Point smoothing hint
	GLenum PointSmoot;
		/// Polygon smoothing hint
	GLenum PolygonSmooth;

private:
	static const char * name;
};


inline grHints::grHints( void )
{
	Fog				= GL_DONT_CARE;
	LineSmoot		= GL_DONT_CARE;
	PerspCorrection	= GL_DONT_CARE;
	PointSmoot		= GL_DONT_CARE;
	PolygonSmooth	= GL_DONT_CARE;
}


inline void grHints::GL( void ) const
{
	glHint(GL_FOG_HINT,Fog);
	glHint(GL_LINE_SMOOTH_HINT,LineSmoot);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,PerspCorrection);
	glHint(GL_POINT_SMOOTH_HINT,PointSmoot);
	glHint(GL_POLYGON_SMOOTH_HINT,PolygonSmooth);
}


inline void grHints::Get( void )
{
	glGet(GL_FOG_HINT,(int *)&Fog);
	glGet(GL_LINE_SMOOTH_HINT,(int *)&LineSmoot);
	glGet(GL_PERSPECTIVE_CORRECTION_HINT,(int *)&PerspCorrection);
	glGet(GL_POINT_SMOOTH_HINT,(int *)&PointSmoot);
	glGet(GL_POLYGON_SMOOTH_HINT,(int *)&PolygonSmooth);
}


#endif
