


/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_MATERIAL
#define GR_SHQN_MATERIAL


#include <vector>
#include "base.h"
#include "node.h"
#include "color.h"
#include "status.h"


using namespace std;

	/** Material definition.
	    Single (direct) material definition.
		This node defines directly the current opengl material (modify
		directly the current opengl status).
		@see opengl.
	  */
class grSingleMaterial : public grNode
{
public:
	inline grSingleMaterial();
	inline void GL() const;

	const char * Name() const;
	void Render( grStatus & status );

		/// Face mode, values: GL_FRONT, GL_BACK, or GL_FRONT_AND_BACK
	GLenum	  face;
		/// Values: (0,128) 0 = no shininess.
	float     shininess;
		/// Ambient light color
	grColor4f ambient;
		/// Prope color
	grColor4f diffuse;
		/// Specular reflection color
	grColor4f specular;
		/// Emissive color
	grColor4f emission;

private:
	static const char * name;
};


	/** Material (color) definition.
	    This node defines the current set of materials and modify
		the traversal status of the current tree.
	    @see opengl, grMaterialBinding, grPrimitive
	 */
class grMaterial : public grNode
{
public:
	inline grMaterial();
	void GL( const int i ) const;

	const char * Name() const;
	void Render( grStatus & status );

		/// Face mode, values: GL_FRONT, GL_BACK, or GL_FRONT_AND_BACK
	GLenum			  face;
		/// Values: (0,128) 0 = no shininess.
	vector<float>     shininess;
		/// Ambient light color
	vector<grColor4f> ambient;
		/// Prope color
	vector<grColor4f> diffuse;
		/// Specular reflection color
	vector<grColor4f> specular;
		/// Emissive color
	vector<grColor4f> emission;

private:
	static const char * name;
};



inline grSingleMaterial::grSingleMaterial()
{
	face = GL_FRONT_AND_BACK;
	shininess = 0.0f;
	ambient  = grColor4f(0.2f, 0.2f, 0.2f, 1.0f);
	diffuse  = grColor4f(0.8f, 0.8f, 0.8f, 1.0f);
	specular = grColor4f(0.0f, 0.0f, 0.0f, 1.0f);
	emission = grColor4f(0.0f, 0.0f, 0.0f, 1.0f);
}


inline void grSingleMaterial::GL() const
{
	glMaterialf(face,GL_SHININESS,shininess);
	glMaterialfv(face,GL_AMBIENT,ambient.color);
	glMaterialfv(face,GL_DIFFUSE,diffuse.color);
	glMaterialfv(face,GL_SPECULAR,specular.color);
	glMaterialfv(face,GL_EMISSION,emission.color);
}


inline grMaterial::grMaterial()
{
	face = GL_FRONT_AND_BACK;
	shininess.push_back(0.0f);
	ambient.push_back( grColor4f(0.2f, 0.2f, 0.2f, 1.0f) );
	diffuse.push_back( grColor4f(0.8f, 0.8f, 0.8f, 1.0f) );
	specular.push_back( grColor4f(0.0f, 0.0f, 0.0f, 1.0f) );
	emission.push_back( grColor4f(0.0f, 0.0f, 0.0f, 1.0f) );
}


#endif