
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V0.9 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

#pragma warning( disable : 4514 4201 )

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <assert.h>


#include "matrix.h"



#if 0
grMatrix & grMatrix::operator = ( const grMatrix & mat ){
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        m[i][j] = mat.a[i][j];
    return *this;
}


grMatrix grMatrix::operator + ( const grMatrix & m ) const {
    grMatrix r;
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        r.a[i][j] = a[i][j] + m.a[i][j];
    return r;
}


grMatrix grMatrix::operator - ( const grMatrix & m ) const {
    grMatrix r;
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        r.a[i][j] = a[i][j] - m.a[i][j];
    return r;
}


grMatrix grMatrix::operator * ( const grMatrix & m ) const {
    grMatrix r;
    int i,j,k;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++){
        Real t = 0.0;
        for(k=0;k<4;k++)
          t += a[i][k] * m.a[k][j];
        r.a[i][j] = t;
      }
    return r;
}


grMatrix grMatrix::operator - ( void ) const {
    grMatrix r;
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        r.a[i][j] = - a[i][j];
    return r;
}


grMatrix grMatrix::operator * ( const Real k ) const {
    grMatrix r;
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        r.a[i][j] = k * a[i][j];
    return r;
}


grMatrix & grMatrix::operator += ( const grMatrix & m ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] += m.a[i][j];
    return *this;
}


grMatrix & grMatrix::operator -= ( const grMatrix & m ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] -= m.a[i][j];
    return *this;
}


grMatrix & grMatrix::operator *= ( const grMatrix & m ) {
    grMatrix r;
    int i,j,k;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++){
        Real t = 0.0;
        for(k=0;k<4;k++)
          t += a[i][k] * m.a[k][j];
        r.a[i][j] = t;
      }
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] = r.a[i][j];
    return *this;
}


grMatrix & grMatrix::operator *= ( const Real k ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j]*= k;
    return *this;
}


grMatrix & grMatrix::SetScalar( const Real k ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i==j) a[i][j] = k;
        else     a[i][j] = 0;
    return *this;
}


grMatrix & grMatrix::SetRotX( const Real alpha ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] = 0;
    a[1][1] = COS(alpha);
    a[1][2] = SIN(alpha);
    a[2][1] = - a[1][2];
    a[2][2] = a[1][1];
    a[0][0] = 1;
    a[3][3] = 1;
    return *this;
}


grMatrix & grMatrix::SetRotY( const Real alpha ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] =0;
    a[0][0] = COS(alpha);
    a[0][2] = SIN(alpha);
    a[2][0] = - a[0][2];
    a[2][2] = a[0][0];
    a[1][1] = 1;
    a[3][3] = 1;
    return *this;
}


grMatrix & grMatrix::SetRotZ( const Real alpha ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] =0;
    a[0][0] = COS(alpha);
    a[0][1] = SIN(alpha);
    a[1][0] = - a[0][1];
    a[1][1] = a[0][0];
    a[2][2] = 1;
    a[3][3] = 1;
    return *this;
}


grMatrix & grMatrix::SetTrans( const Point3D & p ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i==j) a[i][j] = 1;
        else     a[i][j] = 0;
    a[0][3] = p.x;           
    a[1][3] = p.y;
    a[2][3] = p.z;
    return *this;
}


grMatrix & grMatrix::SetTrans( const Real tx, const Real ty, const Real tz){
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i==j) a[i][j] = 1;
        else     a[i][j] = 0;
    a[0][3] = tx;           
    a[1][3] = ty;
    a[2][3] = tz;
    return *this;
}


grMatrix & grMatrix::SetScale( const Point3D & p ) {
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i!=j) a[i][j] = 0;
    a[0][0] = p.x;
    a[1][1] = p.y;
    a[2][2] = p.z;
    a[3][3] = 1;
    return *this;
}
     

grMatrix & grMatrix::SetScale( const Real sx, const Real sy, const Real sz){
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i!=j) a[i][j] = 0;
    a[0][0] = sx;
    a[1][1] = sy;
    a[2][2] = sz;
    a[3][3] = 1;
    return *this;
}


grMatrix & grMatrix::SetScale( const Real k ){
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        if(i==j) a[i][j] = 1;
        else     a[i][j] = 0;
    a[3][3] = k;
    return *this;
}


	// D = distanza osservatore dal piano prospettico
	// d = distanza piano prospettico dall' origine
	
grMatrix & grMatrix::SetProsp( const Real D, const Real d ){
    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] = 0;
    a[0][0] = D;
    a[1][1] = D;
    a[3][2] = 1;
    a[3][3] = d;
    return *this;
}


grMatrix & grMatrix::SetHorto( void ){

    int i,j;
    
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        a[i][j] = 0;
    a[0][0] = 1;
    a[1][1] = 1;
    a[3][3] = 1;
    return *this;
}
#endif
