
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

#pragma warning( disable : 4514 4201 4804 )

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <assert.h>


#include "primitive.h"
#include "mbinding.h"
#include "nbinding.h"
#include "tcbinding.h"
#include "normal.h"
#include "material.h"
#include "texcoord.h"


const char * grPoints::name			="Points";
const char * grLines::name			="Lines";
const char * grLineStrip::name		="LineStrip";
const char * grLineLoop::name		="LineLoop";
const char * grTriangles::name		="Triangles";
const char * grTriangleStrip::name	="TriangleStrip";
const char * grTriangleFan::name	="TriangleFan";
const char * grQuads::name			="Quads";
const char * grQuadStrip::name		="QuadStrip";
const char * grPolygons::name		="Polygons";


grNode * grPrimitive::FindByName( const GLuint name )
{
	if(name==glname)
		return this;
	else
		return 0;
}

const char * grPoints::Name() const
{
	return name;
}


const char * grLines::Name() const
{
	return name;
}


const char * grLineStrip::Name() const
{
	return name;
}


const char * grLineLoop::Name() const
{
	return name;
}


const char * grTriangles::Name() const
{
	return name;
}


const char * grTriangleStrip::Name() const
{
	return name;
}


const char * grTriangleFan::Name() const
{
	return name;
}


const char * grQuads::Name() const
{
	return name;
}


const char * grQuadStrip::Name() const
{
	return name;
}


const char * grPolygons::Name() const
{
	return name;
}

	// Helper functions

static inline void OverAll( grStatus & status )
{
	if(status.material &&
	   status.mbinding==grMaterialBinding::OVERALL)
		status.material->GL(0);
	if(status.normal &&
	   status.nbinding==grNormalBinding::OVERALL)
		status.normal->GL(0);
	if(status.texcoord &&
	   status.tcbinding==grTexCoordBinding::OVERALL)
		status.texcoord->GL(0);
}


void grPoints::Stat( grStatus & status, const int i )
{
	if(status.material)
	{
		if(status.mbinding==grMaterialBinding::PER_VERTEX)
			status.material->GL(i);
	}

	if(status.normal)
	{
		if(status.nbinding==grNormalBinding::PER_VERTEX)
			status.normal->GL(i);
	}

	if(status.texcoord)
	{
		if(status.tcbinding==grTexCoordBinding::PER_VERTEX)
			status.texcoord->GL(i);
	}
}


void grPoints::IStat( grStatus & status, const int i )
{
	if(status.material)
	{
		if(status.mbinding==grMaterialBinding::PER_VERTEX)
			status.material->GL((*this)[i]);
	}

	if(status.normal)
	{
		if(status.nbinding==grNormalBinding::PER_VERTEX)
			status.normal->GL((*this)[i]);
	}

	if(status.texcoord)
	{
		if(status.tcbinding==grTexCoordBinding::PER_VERTEX)
			status.texcoord->GL((*this)[i]);
	}
}


void grLines::Stat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
			if(i%2==0)
				status.material->GL(i/2);
			break;
		case grMaterialBinding::PER_VERTEX:
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
			if(i%2==0)
				status.normal->GL(i/2);
			break;
		case grNormalBinding::PER_VERTEX:
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i);
			break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
			if(i%2==0)
				status.texcoord->GL(i/2);
			break;
		case grTexCoordBinding::PER_VERTEX:
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i);
			break;
		}
	}
}


void grLines::IStat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_VERTEX:
			status.material->GL((*this)[i]);
			break;
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
			if(i%2==0)
				status.material->GL(i/2);
			break;
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_VERTEX:
			status.normal->GL((*this)[i]);
			break;
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
			if(i%2==0)
				status.normal->GL(i/2);
			break;
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i);
			break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_VERTEX:
			status.texcoord->GL((*this)[i]);
			break;
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
			if(i%2==0)
				status.texcoord->GL(i/2);
			break;
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i);
			break;
		}
	}
}


void grLineStrip::Stat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
		case grMaterialBinding::PER_VERTEX:
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
		case grNormalBinding::PER_VERTEX:
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i);
			break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
		case grTexCoordBinding::PER_VERTEX:
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i);
			break;
		}
	}
}


void grLineStrip::IStat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_VERTEX:
			status.material->GL((*this)[i]);
			break;
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_VERTEX:
			status.normal->GL((*this)[i]);
			break;
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i);
			break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_VERTEX:
			status.texcoord->GL((*this)[i]);
			break;
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i);
			break;
		}
	}
}


void grTriangles::Stat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
			if(i%3==0)
				status.material->GL(i/3);
			break;
		case grMaterialBinding::PER_VERTEX:
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
			if(i%3==0)
				status.normal->GL(i/3);
			break;
		case grNormalBinding::PER_VERTEX:
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i); break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
			if(i%3==0)
				status.texcoord->GL(i/3);
			break;
		case grTexCoordBinding::PER_VERTEX:
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i); break;
		}
	}
}


void grTriangles::IStat( grStatus & status, const int i )
{
	if(status.material)
	{
		switch(status.mbinding)
		{
		case grMaterialBinding::PER_VERTEX:
			status.material->GL((*this)[i]);
			break;
		case grMaterialBinding::PER_PART:
		case grMaterialBinding::PER_FACE:
			if(i%3==0)
				status.material->GL(i/3);
			break;
		case grMaterialBinding::PER_CORNER:
			status.material->GL(i);
			break;
		}
	}

	if(status.normal)
	{
		switch(status.nbinding)
		{
		case grNormalBinding::PER_VERTEX:
			status.normal->GL((*this)[i]);
			break;
		case grNormalBinding::PER_PART:
		case grNormalBinding::PER_FACE:
			if(i%3==0)
				status.normal->GL(i/3);
			break;
		case grNormalBinding::PER_CORNER:
			status.normal->GL(i); break;
		}
	}

	if(status.texcoord)
	{
		switch(status.tcbinding)
		{
		case grTexCoordBinding::PER_VERTEX:
			status.texcoord->GL((*this)[i]);
			break;
		case grTexCoordBinding::PER_PART:
		case grTexCoordBinding::PER_FACE:
			if(i%3==0)
				status.texcoord->GL(i/3);
			break;
		case grTexCoordBinding::PER_CORNER:
			status.texcoord->GL(i); break;
		}
	}
}


void grPoints::Render( grStatus & status )	// V1.0
{
	if(status.vertex==0)
		return;

	glBegin(GL_POINTS);

	OverAll(status);

	if(size()==0)
	{
		for(int i=0;i<int(status.vertex->Size());i++)
		{	
			Stat(status,i);
			status.vertex->GL(i);
		}
	}
	else
	{
		for(int i=0;i<int(size());i++)
		{
			IStat(status,i);
			status.vertex->GL((*this)[i]);
		}
	}

	glEnd();
}


void grLines::Render( grStatus & status )	// V1.0
{
	if(status.vertex==0)
		return;

	glBegin(GL_LINES);


	OverAll(status);

	if(size()==0)
	{
		for(int i=0;i<int(status.vertex->Size());i++)
		{	
			Stat(status,i);
			status.vertex->GL(i);
		}
	}
	else
	{
		for(int i=0;i<int(size());i++)
		{
			IStat(status,i);
			status.vertex->GL((*this)[i]);
		}
	}

	glEnd();
}


void grLineStrip::Render( grStatus & status )	// V1.0
{
	if(status.vertex==0)
		return;

	glBegin(GL_LINE_STRIP);

	OverAll(status);

	if(size()==0)
	{
		for(int i=0;i<int(status.vertex->Size());i++)
		{	
			Stat(status,i);
			status.vertex->GL(i);
		}
	}
	else
	{
		for(int i=0;i<int(size());i++)
		{
			if((*this)[i]==-1)
			{
				if(i!=int(size())-1)
				{
					glEnd();
					glBegin(GL_LINE_STRIP);
				}
			}
			else
			{
				IStat(status,i);
				status.vertex->GL((*this)[i]);
			}
		}
	}


	glEnd();
}


void grTriangles::Render( grStatus & status )
{
	if(status.vertex==0)
		return;

	glBegin(GL_TRIANGLES);

	OverAll(status);

	if(size()==0)
	{
		for(int i=0;i<int(status.vertex->Size());i++)
		{
			Stat(status,i);
			status.vertex->GL(i);
		}
	}
	else
	{
		for(int i=0;i<int(size());i++)
		{
			IStat(status,i);
			status.vertex->GL((*this)[i]);
		}
	}
	
	glEnd();
}

void grPoints::Select( grStatus & status )
{
	if(status.vertex==0)
		return;

	if(status.onesel!=this)
	{
		glPushName(glname);
		glBegin(GL_POINTS);
		if(size()==0)
		{
			for(int i=0;i<int(status.vertex->Size());i++)
				status.vertex->GL(i);
		}
		else
		{
			for(int i=0;i<int(size());i++)
				status.vertex->GL((*this)[i]);
		}
		glEnd();
		glPopName();
	}
	else
	{
		if(size()==0)
		{
			for(int i=0;i<int(status.vertex->Size());i++)
			{
				glPushName(i);
				glBegin(GL_POINTS);
				status.vertex->GL(i);
				glEnd();
				glPopName();
			}
		}
		else
		{
			for(int i=0;i<int(size());i++)
			{
				glPushName(i);
				glBegin(GL_POINTS);
				status.vertex->GL((*this)[i]);
				glEnd();
				glPopName();
			}
		}
	}
}

void grLineStrip::Select( grStatus & /*status*/ )
{
	// TODO
}

void grTriangles::Select( grStatus & status )
{
	if(status.vertex==0)
		return;

	if(status.onesel!=this)
	{
		glPushName(glname);
		glBegin(GL_TRIANGLES);

		if(size()==0)
		{
			for(int i=0;i<int(status.vertex->Size());i++)
				status.vertex->GL(i);
		}
		else
		{
			for(int i=0;i<int(size());i++)
				status.vertex->GL((*this)[i]);
		}
		
		glEnd();
		glPopName();
	}
	else
	{
		if(size()==0)
		{
			for(int i=0;i<int(status.vertex->Size())/3;i++)
			{
				glPushName(i);
				glBegin(GL_TRIANGLES);
					status.vertex->GL(i*3+0);
					status.vertex->GL(i*3+1);
					status.vertex->GL(i*3+2);
				glEnd();
				glPopName();
			}
		}
		else
		{
			for(int i=0;i<int(size())/3;i++)
			{
				glPushName(i);
				glBegin(GL_TRIANGLES);
					status.vertex->GL((*this)[i*3+0]);
					status.vertex->GL((*this)[i*3+1]);
					status.vertex->GL((*this)[i*3+2]);
				glEnd();
				glPopName();
			}
		}
	}
}


