
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_PRIMITIVE
#define GR_SHQN_PRIMITIVE

#include <vector>
#include "node.h"
#include "status.h"
#include "vertex.h"


using namespace std;

	/** Abstract base class for all primitive nodes.
	    This is the abstract base class from wich all primitive node
		classes are derived, The primitive are: points, lines, line strips,
		lineloop, triangles, triangle strips, triangle fans, quads,
		quad strips, poligons. Note that all the primitive nodes are
		derived from vector<int>: each int represents a vertex index on the
		current vertex set. If a primitive node is empty, all the current
		node vertex set are used to generate a set of primitives.
		On the complex sets, the value -1 are used as end-set mark, i.e.
		on triangle strip node, a value of -1 marks the end of a strip.
		@see grVertex, opengl.
	 */
class grPrimitive: public grNode, public virtual vector<int>
{
public:
	inline grPrimitive();
	grNode * FindByName( const GLuint name );

protected:
	GLuint glname;
};

	/** @name Primitives
	  */

	//@{

	/// Points set
class grPoints : public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};


	/// Lines set
class grLines : public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Line strips set
class grLineStrip: public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Line loops set.
class grLineLoop: public grPrimitive
{
public:
	void Render( grStatus & status );
	//void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Triangles set
class grTriangles : public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Triangle strips set
class grTriangleStrip : public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Triangle fans set
class grTriangleFan : public grPrimitive
{
public:
	void Render( grStatus & status );
	void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Quads set
class grQuads : public grPrimitive
{
public:
	void Render( grStatus & status );
	//void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Quad strips set
class grQuadStrip : public grPrimitive
{
public:
	void Render( grStatus & status );
	//void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};

	/// Polygons set
class grPolygons : public grPrimitive
{
public:
	void Render( grStatus & status );
	//void Select( grStatus & status );
	const char * Name( void ) const;

private:
	void  Stat( grStatus & status, const int i );
	void IStat( grStatus & status, const int i );
	static const char * name;
};


	// Implementazione di grPrimitive

inline grPrimitive::grPrimitive()
{
	glname = ++grStatus::lastname;
}

	//@}

#endif
