
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_QUADRIC
#define GR_SHQN_QUADRIC


#include "base.h"
#include "node.h"


	/** Abstract base class for all quadric node nodes.
		This is the abstract base class from wich all quadric node
		classes are derived.
	    @see GLU
	 */
class grQuadric : public grNode
{
public:
		/// Default constructor
	inline  grQuadric();
		/// Default destructor
	inline ~grQuadric( void );
		/// see grNode
	const char * Name( void ) const;

	//inline void SetErrorCallBack( void (* cb)( ) );

		/// Values: GLU_FILL GLU_LINE GLU_SILHOUETTE GLU_POINT
	inline void SetDrawStyle( GLenum style );
		/// Values: GLU_NONE GLU_FLAT GLU_SMOOTH
	inline void SetNormalStyle( GLenum style );
		/// Values: GLU_OUTSIDE GLU_INSIDE
	inline void SetOrientation( GLenum orientation );
		/// If enabled texture coordinates are generated automatically
	inline void EnableTexture( bool enable );

protected:
	GLUquadricObj * q;
	static const char * name;
};


inline grQuadric::grQuadric()
{
	q = gluNewQuadric();
	assert(q);
}


inline grQuadric::~grQuadric( )
{
	assert(q);
	gluDeleteQuadric(q);
}


//inline void grQuadric::SetErrorCallBack( void (* /*cb*/)( ) )
//{
	// NOT WORK
	//gluQuadricCallback(q,GLU_ERROR,cb);
//}


inline void grQuadric::SetDrawStyle( GLenum style )
{
	gluQuadricDrawStyle(q,style);
}


inline void grQuadric::SetNormalStyle( GLenum style )
{
	gluQuadricNormals(q,style);
}


inline void grQuadric::SetOrientation( GLenum orientation )
{
	gluQuadricOrientation(q,orientation);
}


inline void grQuadric::EnableTexture( bool enable )
{
	gluQuadricTexture(q,enable);
}



#endif
