
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/

#pragma warning( disable : 4514 4201 4804 )

#include <windows.h>
#include <stdio.h>
#include <limits.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <algorithm>

#include "render.h"
#include "algo.h"

grRender * grRender::glOwner = NULL;


grRender::grRender()
{
	init      = 0;
	scene     = 0;
	ClearMask = GL_COLOR_BUFFER_BIT  | GL_DEPTH_BUFFER_BIT;
}

void grRender::Render()
{
	CheckOwner();
		
	glClear( ClearMask );

	status.Reset();
	glLoadIdentity();

	glTranslatef( viewer.trans[0],viewer.trans[1],viewer.trans[2] );
	glMultMatrixf( (float *)viewer.mat);
	glScalef(     viewer.scale[0],viewer.scale[1],viewer.scale[2] );
	glTranslatef( viewer.rtran[0],viewer.rtran[1],viewer.rtran[2] );

	if(scene!=0)
		scene->Render(status);
	glFinish();

	SwapBuffers(wextra.ghDC);
}


void grRender::Init()
{
	CheckOwner();

	glClear( ClearMask );
	status.Reset();
	if(init!=0)
		init->Render(status);
}


BOOL grRender::OnCreate(void * hwnd)
{
	viewer.Init(10,10);
	glOwner = this;
	return grOnCreate(hwnd,wextra);
}


void grRender::OnDestroy( void * hwnd )
{
	CheckOwner();
	grOnDestroy(hwnd,wextra);
	glOwner = NULL;
}


void grRender::OnSize( void * hwnd, int w, int h )
{
	CheckOwner();
	grOnSize(hwnd,wextra,w,h);
	viewer.Resize(w,h);
	//glClear( ClearMask );			// NECESSARIO SE CLEAR ANTICIPATA!
}


void grRender::PortSize( int & w, int & h )
{
	w = wextra.viewport[2] - wextra.viewport[0];
	h = wextra.viewport[3] - wextra.viewport[1];
}


void grRender::ReadSubPixel( int x, int y, int w, int h, GLenum format, GLenum type, GLvoid * mem )
{
	CheckOwner();
		
	glClear( ClearMask );
	
	status.Reset();
	glLoadIdentity();

	glTranslated(0.0, 0.0, -15);

	glTranslatef( viewer.trans[0],viewer.trans[1],viewer.trans[2] );
	glMultMatrixf( (float *)viewer.mat);
	glScalef(     viewer.scale[0],viewer.scale[1],viewer.scale[2] );
	glTranslatef( viewer.rtran[0],viewer.rtran[1],viewer.rtran[2] );

	if(scene!=0)
		scene->Render(status);
	glFinish();

	glReadPixels(x,y,w,h,format,type,mem);

	SwapBuffers(wextra.ghDC);
	//glClear( ClearMask );
}

void grRender::ReadPixel( GLenum format, GLenum type, GLvoid * mem )
{
	int w = wextra.viewport[2] - wextra.viewport[0];
	int h = wextra.viewport[3] - wextra.viewport[1];

	ReadSubPixel(0,0,w,h,format,type,mem);
}

bool grRender::ScreenToWorld( const int sx, const int sy, Coord3d & p )
{
	CheckOwner();

	GLdouble modelm[16];
	glGetDoublev(GL_MODELVIEW_MATRIX,modelm);

	GLfloat fz;
	glReadPixels(sx,sy,1,1,GL_DEPTH_COMPONENT , GL_FLOAT, &fz);
	if(fz==0.0)
		return false;

	GLdouble px,py,pz;

	if( gluUnProject( GLdouble(sx), GLdouble(sy), fz, modelm, wextra.projMatrix, 
		              wextra.viewport, &px, &py, &pz ) == GL_FALSE )
		return false;
	
	p.x = px;
	p.y = py;
	p.z = pz;

	return true;
}

void grRender::PickNodes( int x, int y, double wx, double wy, grPickResult & pr )
{
	CheckOwner();

		// Definizione del buffer di selezione
	const GLsizei MAXSBUF = 512;
	GLuint sbuf[MAXSBUF];
	GLint  buflen;

		// Preparativi generali
	glSelectBuffer( MAXSBUF, sbuf );	// Seleziona il buffer di picking
	glRenderMode( GL_SELECT );			// Modalita' select
	//glLoadName(0);						// Carica il nome di default
	
		// Rovescio la y
	y =  wextra.viewport[3] - wextra.viewport[1] - 1 - y;

		// Preparazione matrice di proiezione
	glMatrixMode( GL_PROJECTION ); 
	glLoadIdentity(); 
	gluPickMatrix(x, y, wx,wy, wextra.viewport); 
	gluPerspective( wextra.prospAngle, wextra.aspect, wextra.nearPlane, wextra.farPlane );
	glMatrixMode( GL_MODELVIEW );

		// Rendering speciale
	glClear( ClearMask );				// Ristora i buffer originali
	status.Reset();
	glLoadIdentity();

	glTranslatef( viewer.trans[0],viewer.trans[1],viewer.trans[2] );
	glMultMatrixf( (float *)viewer.mat);
	glScalef(     viewer.scale[0],viewer.scale[1],viewer.scale[2] );
	glTranslatef( viewer.rtran[0],viewer.rtran[1],viewer.rtran[2] );

	if(scene!=0)
		scene->Select(status);

	buflen = glRenderMode( GL_RENDER );	// Chiusura modalita' selezione

		// Parsing dello stack
	pr.resize(buflen);
	int j = 0;
	for(int i=0;i<buflen;++i)
	{
		int n = sbuf[j++];				// Namestack size
		pr[i].minDepth = sbuf[j++]/double(UINT_MAX);
		pr[i].maxDepth = sbuf[j++]/double(UINT_MAX);
		pr[i].nameStack.resize(n);
		for(int k=0;k<n;++k)
			pr[i].nameStack[k] = FindByName(sbuf[j++]);
	}
		// Ordinamento per profondita' 
	sort(pr.begin(),pr.end());
	
		// Restoring della matrice originale
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();  
	gluPerspective( wextra.prospAngle, wextra.aspect, wextra.nearPlane, wextra.farPlane );
	glMatrixMode(GL_MODELVIEW);  
}


unsigned int grRender::PickInNode( int x, int y, double wx, double wy, grNode * node )
{
	CheckOwner();

			// Definizione del buffer di selezione
	const GLsizei MAXSBUF = 4096;
	GLuint sbuf[MAXSBUF];
	GLint  buflen;

		// Preparativi generali
	glSelectBuffer( MAXSBUF, sbuf );	// Seleziona il buffer di picking
	glRenderMode( GL_SELECT );			// Modalita' select
	
		// Rovescio la y
	y =  wextra.viewport[3] - wextra.viewport[1] - 1 - y;

		// Preparazione matrice di proiezione
	glMatrixMode( GL_PROJECTION ); 
	glLoadIdentity(); 
	gluPickMatrix(x, y, wx,wy, wextra.viewport); 
	gluPerspective( wextra.prospAngle, wextra.aspect, wextra.nearPlane, wextra.farPlane );
	glMatrixMode( GL_MODELVIEW );

		// Rendering speciale
	glClear( ClearMask );				// Ristora i buffer originali
	status.Reset();
	status.onesel = node;
	glLoadIdentity();

	glTranslatef( viewer.trans[0],viewer.trans[1],viewer.trans[2] );
	glMultMatrixf( (float *)viewer.mat);
	glScalef(     viewer.scale[0],viewer.scale[1],viewer.scale[2] );
	glTranslatef( viewer.rtran[0],viewer.rtran[1],viewer.rtran[2] );

	if(scene!=0)
		scene->Select(status);

	buflen = glRenderMode( GL_RENDER );	// Chiusura modalita' selezione

	unsigned int ret = (unsigned int)(-1);
	
		// Parsing dello stack e ordinamento
	if(buflen!=0)
	{
		unsigned int mind = 0;

		int j = 0;
		for(int i=0;i<buflen;++i)
		{
			int n = sbuf[j++];				// Namestack size
			unsigned int curd = sbuf[j++];	// minDepth
			++j;							// maxDepth
			unsigned int curn = 0;			// current name
			for(int k=0;k<n;++k)
				curn = sbuf[j++];
			if(i==0 || mind > curd )
			{
				mind = curd;
				ret  = curn;
			}
		}
	}
	
		// Restoring della matrice originale
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();  
	gluPerspective( wextra.prospAngle, wextra.aspect, wextra.nearPlane, wextra.farPlane );
	glMatrixMode(GL_MODELVIEW);  

	return ret;
}


bool grRender::Pick( int x, int y, double wx, double wy, grNode * & node, int & prim )
{
	CheckOwner();

	grPickResult pr;
	PickNodes(x,y,wx,wy,pr);
	if(pr.size()!=0)
	{
		node = pr[0].nameStack[pr[0].nameStack.size()-1];
		prim = PickInNode(x,y,wx,wy,node);
		return true;
	}
	else
	{
		node = 0;
		prim = -1;
		return false;
	}
}


grNode * grRender::FindByName( GLuint name )
{
	if(scene!=0)
		return scene->FindByName(name);
	else
		return 0;
}

void grRender::grViewAll( const grVertex3f & vert)
{
	grScalef       s;
	grTranslationf t;

	::grViewAll(vert,s,t);
	viewer.scale[0] = s.x;
	viewer.scale[1] = s.y;
	viewer.scale[2] = s.z;
	viewer.rtran[0] = t.x;
	viewer.rtran[1] = t.y;
	viewer.rtran[2] = t.z;
}

void grRender::grViewAll( const Boxd & boundbox )
{
	grScalef       s;
	grTranslationf t;

	::grViewAll(boundbox,s,t);
	viewer.scale[0] = s.x;
	viewer.scale[1] = s.y;
	viewer.scale[2] = s.z;
	viewer.rtran[0] = t.x;
	viewer.rtran[1] = t.y;
	viewer.rtran[2] = t.z;
}

void grRender::grViewAll( const Boxf & boundbox )
{
	grScalef       s;
	grTranslationf t;

	::grViewAll(boundbox,s,t);
	viewer.scale[0] = s.x;
	viewer.scale[1] = s.y;
	viewer.scale[2] = s.z;
	viewer.rtran[0] = t.x;
	viewer.rtran[1] = t.y;
	viewer.rtran[2] = t.z;
}

void grRender::StartDrag( int x, int y )
{
	CheckOwner();


	GLint vport[4];
	glGetIntegerv(GL_VIEWPORT, vport );

	curHeight = vport[3];

	y = curHeight - y;

	ox = dx = x;
	oy = dy = y;

	glGetIntegerv( GL_DRAW_BUFFER, &oldDrawBuffer );
	glGetIntegerv( GL_DEPTH_FUNC, &oldDepthFunc );
	glGetIntegerv( GL_LINE_STIPPLE_PATTERN, & oldLineStippleP );
	glGetIntegerv( GL_LINE_STIPPLE_REPEAT , & oldLineStippleR );
	glDrawBuffer( GL_FRONT );
	glDepthFunc( GL_ALWAYS );
	glLineStipple( 1, 0xF6F6 );

	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	glOrtho( 0, vport[2], 0, vport[3], 0, 1 );

	GLfloat white[4] = {1,1,1,1};

	glGetMaterialfv( GL_FRONT, GL_EMISSION ,oldEColor );
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, white );
	BlendingIsE = glIsEnabled( GL_BLEND );
	glEnable( GL_BLEND );
	
	glGetIntegerv( GL_BLEND_SRC, & BlendFun1 );
	glGetIntegerv( GL_BLEND_DST, & BlendFun2 );
	glBlendFunc( GL_ONE_MINUS_DST_COLOR, GL_ZERO );
}

void grRender::Drag( int x, int y )
{
	CheckOwner();

	y = curHeight - y;

	if(x!=dx || y!=dy )
	{
		glBegin(GL_LINE_LOOP);
			glVertex2i( ox,oy );
			glVertex2i( ox,dy );
			glVertex2i( dx,dy );
			glVertex2i( dx,oy );
		glEnd();

		dx = x;
		dy = y;

		glBegin(GL_LINE_LOOP);
			glVertex2i( ox,oy );
			glVertex2i( ox,dy );
			glVertex2i( dx,dy );
			glVertex2i( dx,oy );
		glEnd();

		glFinish();
	}
}

void grRender::EndDrag( int x, int y )
{
	CheckOwner();

	y = curHeight - y;

	glBegin(GL_LINE_LOOP);
		glVertex2i( ox,oy );
		glVertex2i( ox,dy );
		glVertex2i( dx,dy );
		glVertex2i( dx,oy );
	glEnd();

	dx = x;
	dy = y;

	glDrawBuffer( oldDrawBuffer );
	glDepthFunc( oldDepthFunc );
	glLineStipple( oldLineStippleR, oldLineStippleP );

	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, oldEColor );
	
	if( !BlendingIsE )
		glDisable( GL_BLEND );
	glBlendFunc( BlendFun1, BlendFun2 );

	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
}

void grRender::GetDragRect( int * rect )
{
	rect[0] = ox;
	rect[1] = oy;
	rect[2] = dx;
	rect[3] = dy;
}