
/*****************************************************************************
 * gcrLib : Graphics Rocchini's Library V1.0 (the nth OpenGL wrapper)        *
 *					                                                         *
 * Copyright(C) 1999 by Claudio Rocchini. All rights reserved.               *
 *					                                                         *
 * Permission  to use, copy, modify, distribute  and sell this  software and *
 * its documentation for any purpose is hereby granted without fee, provided *
 * that  the above copyright notice appear  in all copies and that both that *
 * copyright   notice  and  this  permission  notice  appear  in  supporting *
 * documentation. the author makes  no representations about the suitability *
 * of this software for any purpose. It is provided  "as is" without express *
 * or implied warranty.                                                      *
 *					                                                         *
 *****************************************************************************/


#ifndef GR_SHQN_TREE
#define GR_SHQN_TREE


#include <vector>
#include "base.h"
#include "node.h"
#include "status.h"


using namespace std;

	/** Group node.
	    This node defines a group of nodes. Is contains an ordered list of
		child nodes (a STL vector of grNode *).
		The order of the child nodes represents the traversal order
		for all operation. This node also save the traversal status before
		traversing its children ad restore it after traversing them. The grTree node
		provides caching via opengl lists: the user may call the function Compile to
		chaching the contents of the node (the caching is not automatic!)
		@see grNode, grStatus, grRender, STL
	 */

class grTree : public grNode, public vector<grNode *>
{
public:
		/// Standard constructor
	inline  grTree();
		/// Standard destructor
	inline ~grTree();
		/// Generate a opengl display list for the node
	void Compile();
		/// see grNode
	const char * Name() const;
	void Render( grStatus & status );
	void Select( grStatus & status );
	grNode * FindByName( const GLuint name );

private:
	int    list;
	GLuint glname;
	grStatus bufstatus;

	static const char * name;
};


inline grTree::grTree()
{
	list = -1;
	glname = ++grStatus::lastname;
}


inline grTree::~grTree()
{
	if(list!=-1)
		glDeleteLists(list,1);
}



#endif